package com.qs.mp.web.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.qs.mp.common.constant.HttpStatus;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.PageDomain;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.core.page.TableSupport;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.common.utils.DateUtils;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.common.utils.sql.SqlUtil;
import com.qs.mp.core.domain.LoginUser;
import java.beans.PropertyEditorSupport;
import java.util.Date;
import java.util.List;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;

/**
 * web层通用数据处理
 *
 * @author ygp
 */
public class BaseController {

  protected final Logger logger = LoggerFactory.getLogger(this.getClass());

  /**
   * 将前台传递过来的日期格式的字符串，自动转化为Date类型
   */
  @InitBinder
  public void initBinder(WebDataBinder binder) {
    // Date 类型转换
    binder.registerCustomEditor(Date.class, new PropertyEditorSupport() {
      @Override
      public void setAsText(String text) {
        setValue(DateUtils.parseDate(text));
      }
    });
  }

  /**
   * 设置请求分页数据
   */
  protected void startPage() {
    PageDomain pageDomain = TableSupport.buildPageRequest();
    Integer pageNum = pageDomain.getPageNum();
    Integer pageSize = pageDomain.getPageSize();
    if (StringUtils.isNotNull(pageNum) && StringUtils.isNotNull(pageSize)) {
      String orderBy = SqlUtil.escapeOrderBySql(pageDomain.getOrderBy());
      PageHelper.startPage(pageNum, pageSize, orderBy);
    }
  }

  /**
   * 设置请求排序数据
   */
  protected void startOrderBy() {
    PageDomain pageDomain = TableSupport.buildPageRequest();
    if (StringUtils.isNotEmpty(pageDomain.getOrderBy())) {
      String orderBy = SqlUtil.escapeOrderBySql(pageDomain.getOrderBy());
      PageHelper.orderBy(orderBy);
    }
  }

  /**
   * 响应请求分页数据
   */
  @SuppressWarnings({"rawtypes", "unchecked"})
  protected TableDataInfo getDataTable(List<?> list) {
    TableDataInfo rspData = new TableDataInfo();
    rspData.setCode(AjaxResult.Type.SUCCESS.value());
    rspData.setMsg("查询成功");
    rspData.setRows(list);
    rspData.setTotal(new PageInfo(list).getTotal());
    return rspData;
  }

  protected TableDataInfo getErrorDataTable(String msg) {
    TableDataInfo rspData = new TableDataInfo();
    rspData.setCode(AjaxResult.Type.ERROR.value());
    rspData.setMsg(msg);
    rspData.setRows(null);
    rspData.setTotal(0);
    return rspData;
  }


  /**
   * 返回成功
   */
  public AjaxResult success() {
    return AjaxResult.success();
  }

  /**
   * 返回成功消息
   */
  public AjaxResult success(String message) {
    return AjaxResult.success(message);
  }

  /**
   * 返回成功
   */
  public AjaxResult success(String msg, Object data) {
    return AjaxResult.success(msg, data);
  }

  public AjaxResult success(Object data) {
    return AjaxResult.success(data);
  }


  /**
   * 登陆超时
   */
  public AjaxResult timeout() {
    return AjaxResult.timeout();
  }


  /**
   * 登陆超时
   */
  public AjaxResult timeout(String msg) {
    return AjaxResult.timeout(msg);
  }


  /**
   * 返回失败消息
   */
  public AjaxResult error() {
    return AjaxResult.error();
  }


  /**
   * 返回失败消息
   */
  public AjaxResult error(String message) {
    return AjaxResult.error(message);
  }

  /**
   * 返回成功
   */
  public AjaxResult error(String msg, Object data) {
    return AjaxResult.error(msg, data);
  }

  /**
   * 返回错误码消息
   */
  public AjaxResult error(AjaxResult.Type type, String message) {
    return new AjaxResult(type, message);
  }


  public AjaxResult error(int code, String message) {
    return new AjaxResult(code, message, null);
  }

  public AjaxResult error(ErrorCodeEnum errorCodeEnum) {
    return new AjaxResult(errorCodeEnum.getCode(), errorCodeEnum.getMsg(), null);
  }


  /**
   * 响应返回结果
   *
   * @param rows 影响行数
   * @return 操作结果
   */
  protected AjaxResult toAjax(int rows) {
    return rows > 0 ? AjaxResult.success() : AjaxResult.error();
  }


  /**
   * 响应返回结果
   *
   * @param rows 影响行数
   * @return 操作结果
   */
  protected AjaxResult toAjax(int rows, String msg) {
    return rows > 0 ? success(msg) : error(msg);
  }

  /**
   * 响应返回结果
   *
   * @param rows 影响行数
   * @return 操作结果
   */
  protected AjaxResult toAjax(int rows, String msg, Object data) {
    return rows > 0 ? success(msg, data) : error(msg, data);
  }


  /**
   * 响应返回结果
   *
   * @param result 结果
   * @return 操作结果
   */
  protected AjaxResult toAjax(boolean result) {
    return result ? success() : error();
  }

  /**
   * 响应返回结果
   *
   * @param result 结果
   * @return 操作结果
   */
  protected AjaxResult toAjax(boolean result, String msg) {
    return result ? success(msg) : error(msg);
  }

  /**
   * 响应返回结果
   *
   * @param result 结果
   * @return 操作结果
   */
  protected AjaxResult toAjax(boolean result, String msg, Object data) {
    return result ? success(msg, data) : error(msg, data);
  }

  /**
   * 响应返回结果
   *
   * @return 操作结果
   */
  protected AjaxResult toAjax(AjaxResult.Type type, String msg, Object data) {
    return new AjaxResult(type, msg, data);
  }


  public Map<String, Object> getReturnJson(Map<String, Object> rsMap, int code, String msg,
      Object obj) {
    rsMap.put("code", code);
    rsMap.put("msg", msg);
    if (null != obj) {
      rsMap.put("data", obj);
    }
    logger.info("result data:" + rsMap);
    return rsMap;
    // return JSONObject.fromObject(rsMap).toString();
  }

  /**
   * 页面跳转
   */
  public String redirect(String url) {
    return StringUtils.format("redirect:{}", url);
  }

}
